
//Programme  : RFID - Controle d'acces intelligent
//Autor : F.LASNE 2015
/*
 * ----------------------------------------------------------------------------
 * This is a MFRC522 library example; see https://github.com/miguelbalboa/rfid
 * for further details and other examples.
 * 
 * NOTE: The library file MFRC522.h has a lot of useful info. Please read it.
 * 
 * Released into the public domain.
 * ----------------------------------------------------------------------------
 * 
 * Typical pin layout used:
 * -----------------------------------------------------------------------------------------
 *             MFRC522      Arduino       Arduino   Arduino    Arduino          Arduino
 *             Reader/PCD   Uno           Mega      Nano v3    Leonardo/Micro   Pro Micro
 * Signal      Pin          Pin           Pin       Pin        Pin              Pin
 * -----------------------------------------------------------------------------------------
 * RST/Reset   RST          9             5         D9         RESET/ICSP-5     RST
 * SPI SS      SDA(SS)      10            53        D10        10               10
 * SPI MOSI    MOSI         11 / ICSP-4   51        D11        ICSP-4           16
 * SPI MISO    MISO         12 / ICSP-1   50        D12        ICSP-1           14
 * SPI SCK     SCK          13 / ICSP-3   52        D13        ICSP-3           15
 */
 
#include <SPI.h>
#include <MFRC522.h>
#include <LiquidCrystal.h>
#include <Bridge.h>
#include <Process.h>
#include <SoftwareSerial.h>


// Pins used to connect the display to the Arduino
LiquidCrystal lcd(7, 6, 5, 4, 3, 2);

// Pins used to connect the Bluetooth module HC06
SoftwareSerial mySerial(10, 11); // RX, TX
String messageRecu;


// Pins used to connect the RFID
#define RST_PIN 8 
#define SS_PIN 9

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance.
 
Process date; //gestion du temps Arduino.

void setup() 
{
  Serial.begin(9600);                             // Initialize serial communications with the PC
//  while (!Serial);                                // Do nothing if no serial port is opened (added for Arduinos based on ATMEGA32U4)
  mySerial.begin(9600);                           // Rajout d'un port com pour lecteur bluetooth (pas serial1 a cause du bridge)
  mySerial.println("000");
  SPI.begin();                                    // Init SPI bus
//  Serial.println("SPI OK ...");
  Bridge.begin();                                 // initialisation mode Bridge
//  Serial.println("BRIDGE OK ...");
  mfrc522.PCD_Init();                             // Init MFRC522
  mfrc522.PCD_SetAntennaGain(mfrc522.RxGain_max); //Set Antenna Gain to Max- this will increase reading distance
  
  // initialize digital pin 12 et 13 as an output.
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);
  
  /*---Init temp ---*/
  if (!date.running())  {
      date.begin("date");
      date.addParameter("+%T");
      date.run();
  }
  /*--- FIN init temp ---*/
    
  Serial.println("Lecture Carte ...");
  Serial.println();
  lcd.begin(16, 2);  
  mensageminicial();
}
 
void loop() 
{
  digitalWrite(12, LOW);    // turn the LED RED OFF
  digitalWrite(13, LOW);    // turn the LED GREEN OFF

  // if (mySerial.available() > 0) {serialA = mySerial.read();Serial.println(serialA);}

      while(mySerial.available())
      {
       delay(3);
       char c = mySerial.read();
       messageRecu += c;
      }
      if (messageRecu.length() >0)
      {
       Serial.println(messageRecu);
        if (messageRecu == "CMD RED=ON\n")     
          {digitalWrite(13, HIGH);
          delay(3000);
          Serial.println("led on");
          }
        
        messageRecu="";
      }
    
    
  // Look for new cards
  if ( ! mfrc522.PICC_IsNewCardPresent()) 
  {
    return;
  }
  
  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) 
  {
    return;
  }
  

   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("Verification");

  //lecture UID du Badge
  //Serial.print("UID du tag :");
  String contenu= "";
  //byte lettre;
  for (byte i = 0; i < mfrc522.uid.size; i++) 
  {
     //Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? "0" : "");
     //Serial.print(mfrc522.uid.uidByte[i], HEX);
     contenu.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? "0" : ""));
     contenu.concat(String(mfrc522.uid.uidByte[i], HEX));
  }
 
  contenu.toUpperCase();
  Serial.println("Le GUID est:");
  Serial.print(contenu);
  Serial.println();
  Serial.println("Verification ...");
  verifRFID(contenu);
  
} 
 
 
void verifRFID(String IDrfid){
  Process verif;
  verif.begin("python");
  verif.addParameter("/mnt/sda1/RFID/valid.py");
  verif.addParameter(IDrfid); // Appel de la fonction en python valid.py
  verif.run();
  while (verif.available()){
    String resultatPython = verif.readString(); // lecture du resultat de la fonction
    resultatPython.trim(); // 0 ou 1 ou 2 en string
    
    String val = resultatPython.substring(0,1);
    Serial.println(val);
    
    if(val == "0"){                             // badge RFID non reconnu
      Serial.println("Acces Refuse !");
      Serial.println();
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("NON VALIDE");
      lcd.setCursor(0,1);
      lcd.print("Acces REFUSE");
      digitalWrite(12, HIGH);                    // led rouge allumée
      delay(3000);
      mensageminicial();
      break;
      
      
    }
    else if (val == "1"){                         //Badge verifié et OK
    
      Serial.println("Acces Autorise");
      resultatPython = resultatPython.substring(5,(resultatPython.length()-5));
      Serial.println(resultatPython);
      Serial.println();
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Bonjour M ");
      lcd.print(resultatPython);
      lcd.setCursor(0,1);
      lcd.print("Ouverture porte!");
      digitalWrite(13, HIGH);                   // LED vert allumée
      delay(3000);
      controle(IDrfid);
      mensageminicial();
      break; 
    }
    else {                                     //Badge verifié OK mais pas autorisé
    
      Serial.println("Acces Refusé");
      resultatPython = resultatPython.substring(5,(resultatPython.length()-5));
      Serial.println(resultatPython);
      Serial.println();
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Bonjour M ");
      lcd.print(resultatPython);
      lcd.setCursor(0,1);
      lcd.print("NON Autorise");
      digitalWrite(12, HIGH);                 // LED rouge allumée
      delay(3000);
      mensageminicial();
      break; 
    }
  }
 }
 
void controle(String IDrfid){
  Serial.println("ajout contrôle");
  Process control;
  control.begin("python");
  control.addParameter("/mnt/sda1/RFID/control.py");
  control.addParameter(IDrfid); // Appel de la fonction en python controle qui ajoute l'heure de sortie
  control.run();
} 
 
void mensageminicial()
{
  lcd.clear();
  lcd.print("Passer le badge");  
  lcd.setCursor(0,1);
  lcd.print("sur le lecteur");  
}
